@@ -1,17 +1,19 @@
-# Copyright (C) 2018-2020 The Software Heritage developers
+# Copyright (C) 2018-2021 The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import logging
 from typing import Iterable
 
 from confluent_kafka import Consumer, Producer
 import pytest
 
 from swh.journal.pytest_plugin import assert_all_objects_consumed, consume_messages
-from swh.journal.tests.journal_data import TEST_OBJECTS
+from swh.journal.writer import model_object_dict_sanitizer
 from swh.journal.writer.kafka import KafkaDeliveryError, KafkaJournalWriter
-from swh.model.model import Directory, Release, Revision
+from swh.model.model import BaseModel, Directory, Release, Revision
+from swh.model.tests.swh_model_data import TEST_OBJECTS
 
 
 def test_kafka_writer(
@@ -20,10 +22,11 @@ def test_kafka_writer(
     consumer: Consumer,
     privileged_object_types: Iterable[str],
 ):
-    writer = KafkaJournalWriter(
+    writer = KafkaJournalWriter[BaseModel](
         brokers=[kafka_server],
         client_id="kafka_writer",
         prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
         anonymize=False,
     )
 
@@ -46,6 +49,9 @@ def test_kafka_writer(
             )
     for key, obj_dict in consumed_messages["release"]:
         obj = Release.from_dict(obj_dict)
+        # author is optional for release
+        if obj.author is None:
+            continue
         for person in (obj.author,):
             assert not (
                 len(person.fullname) == 32
@@ -60,10 +66,11 @@ def test_kafka_writer_anonymized(
     consumer: Consumer,
     privileged_object_types: Iterable[str],
 ):
-    writer = KafkaJournalWriter(
+    writer = KafkaJournalWriter[BaseModel](
         brokers=[kafka_server],
         client_id="kafka_writer",
         prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
         anonymize=True,
     )
 
@@ -88,6 +95,9 @@ def test_kafka_writer_anonymized(
             )
     for key, obj_dict in consumed_messages["release"]:
         obj = Release.from_dict(obj_dict)
+        # author is optional for release
+        if obj.author is None:
+            continue
         for person in (obj.author,):
             assert (
                 len(person.fullname) == 32
@@ -96,9 +106,7 @@ def test_kafka_writer_anonymized(
             )
 
 
-def test_write_delivery_failure(
-    kafka_prefix: str, kafka_server: str, consumer: Consumer
-):
+def test_write_delivery_failure(kafka_prefix: str, kafka_server: str):
     class MockKafkaError:
         """A mocked kafka error"""
 
@@ -117,7 +125,10 @@ def test_write_delivery_failure(
 
     kafka_prefix += ".swh.journal.objects"
     writer = KafkaJournalWriterFailDelivery(
-        brokers=[kafka_server], client_id="kafka_writer", prefix=kafka_prefix,
+        brokers=[kafka_server],
+        client_id="kafka_writer",
+        prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
     )
 
     empty_dir = Directory(entries=())
@@ -131,9 +142,7 @@ def test_write_delivery_failure(
     assert delivery_failure.code == "SWH_MOCK_ERROR"
 
 
-def test_write_delivery_timeout(
-    kafka_prefix: str, kafka_server: str, consumer: Consumer
-):
+def test_write_delivery_timeout(kafka_prefix: str, kafka_server: str):
 
     produced = []
 
@@ -144,10 +153,11 @@ def test_write_delivery_timeout(
         def produce(self, **kwargs):
             produced.append(kwargs)
 
-    writer = KafkaJournalWriter(
+    writer = KafkaJournalWriter[BaseModel](
         brokers=[kafka_server],
         client_id="kafka_writer",
         prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
         flush_timeout=1,
         producer_class=MockProducer,
     )
@@ -163,3 +173,76 @@ def test_write_delivery_timeout(
     delivery_failure = exc.value.delivery_failures[0]
     assert delivery_failure.key == empty_dir.id
     assert delivery_failure.code == "SWH_FLUSH_TIMEOUT"
+
+
+class MockBufferErrorProducer(Producer):
+    """A Kafka producer that returns a BufferError on the `n_buffererrors`
+    first calls to produce."""
+
+    def __init__(self, *args, **kwargs):
+        self.n_buffererrors = kwargs.pop("n_bufferrors", 0)
+        self.produce_calls = 0
+
+        super().__init__(*args, **kwargs)
+
+    def produce(self, **kwargs):
+        self.produce_calls += 1
+        if self.produce_calls <= self.n_buffererrors:
+            raise BufferError("Local: Queue full")
+
+        self.produce_calls = 0
+        return super().produce(**kwargs)
+
+
+def test_write_BufferError_retry(kafka_prefix: str, kafka_server: str, caplog):
+    writer = KafkaJournalWriter[BaseModel](
+        brokers=[kafka_server],
+        client_id="kafka_writer",
+        prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
+        flush_timeout=1,
+        producer_class=MockBufferErrorProducer,
+    )
+
+    writer.producer.n_buffererrors = 4
+
+    empty_dir = Directory(entries=())
+
+    caplog.set_level(logging.DEBUG, "swh.journal.writer.kafka")
+    writer.write_addition("directory", empty_dir)
+    records = []
+    for record in caplog.records:
+        if "BufferError" in record.getMessage():
+            records.append(record)
+
+    assert len(records) == writer.producer.n_buffererrors
+
+
+def test_write_BufferError_give_up(kafka_prefix: str, kafka_server: str, caplog):
+    writer = KafkaJournalWriter[BaseModel](
+        brokers=[kafka_server],
+        client_id="kafka_writer",
+        prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
+        flush_timeout=1,
+        producer_class=MockBufferErrorProducer,
+    )
+
+    writer.producer.n_buffererrors = 5
+
+    empty_dir = Directory(entries=())
+
+    with pytest.raises(KafkaDeliveryError):
+        writer.write_addition("directory", empty_dir)
+
+
+def test_write_addition_errors_without_unique_key(kafka_prefix: str, kafka_server: str):
+    writer = KafkaJournalWriter[BaseModel](
+        brokers=[kafka_server],
+        client_id="kafka_writer",
+        prefix=kafka_prefix,
+        value_sanitizer=model_object_dict_sanitizer,
+    )
+
+    with pytest.raises(NotImplementedError):
+        writer.write_addition("BaseModel", BaseModel())
