@@ -4,12 +4,14 @@
 # See top-level LICENSE file for more information
 
 from collections import OrderedDict
+from datetime import datetime, timedelta, timezone
 import itertools
 from typing import Iterable
 
-from swh.journal import serializers
+import pytest
 
-from .conftest import TEST_OBJECTS
+from swh.journal import serializers
+from swh.model.tests.swh_model_data import TEST_OBJECTS
 
 
 def test_key_to_kafka_repeatable():
@@ -30,18 +32,11 @@ def test_key_to_kafka_repeatable():
         assert key == serializers.key_to_kafka(d)
 
 
-def test_get_key():
-    """Test whether get_key works on all our objects"""
-    for object_type, objects in TEST_OBJECTS.items():
-        for obj in objects:
-            assert serializers.object_key(object_type, obj) is not None
-
-
 def test_pprint_key():
     """Test whether get_key works on all our objects"""
     for object_type, objects in TEST_OBJECTS.items():
         for obj in objects:
-            key = serializers.object_key(object_type, obj)
+            key = obj.unique_key()
             pprinted_key = serializers.pprint_key(key)
             assert isinstance(pprinted_key, str)
 
@@ -66,7 +61,7 @@ def test_kafka_to_key():
     ]
     for object_type, objects in TEST_OBJECTS.items():
         for obj in objects:
-            key = serializers.object_key(object_type, obj)
+            key = obj.unique_key()
             keys.append(key)
 
     for key in keys:
@@ -74,3 +69,47 @@ def test_kafka_to_key():
         v = serializers.kafka_to_key(ktk)
 
         assert v == key
+
+
+# limits of supported int values by msgpack
+MININT = -(2 ** 63)
+MAXINT = 2 ** 64 - 1
+
+intvalues = [
+    MININT * 2,
+    MININT - 1,
+    MININT,
+    MININT + 1,
+    -10,
+    0,
+    10,
+    MAXINT - 1,
+    MAXINT,
+    MAXINT + 1,
+    MAXINT * 2,
+]
+
+
+@pytest.mark.parametrize("value", intvalues)
+def test_encode_int(value):
+    assert serializers.kafka_to_value(serializers.value_to_kafka(value)) == value
+
+
+datevalues = [
+    datetime.now(tz=timezone.utc),
+    datetime.now(tz=timezone(timedelta(hours=-23, minutes=-59))),
+    datetime.now(tz=timezone(timedelta(hours=23, minutes=59))),
+    datetime(1, 1, 1, 1, 1, tzinfo=timezone.utc),
+    datetime(2100, 1, 1, 1, 1, tzinfo=timezone.utc),
+]
+
+
+@pytest.mark.parametrize("value", datevalues)
+def test_encode_datetime(value):
+    assert serializers.kafka_to_value(serializers.value_to_kafka(value)) == value
+
+
+@pytest.mark.parametrize("value", datevalues)
+def test_encode_datetime_bw(value):
+    bwdate = {b"swhtype": "datetime", b"d": value.isoformat()}
+    assert serializers.kafka_to_value(serializers.value_to_kafka(bwdate)) == value
